from krex.utils.common import Common
from krex.utils.helpers import generate_timestamp
from ._http_manager import HTTPManager
from .endpoints.order import SpotOrder


class TradeHTTP(HTTPManager):
    async def place_spot_order(
        self,
        product_symbol: str,
        orderQty: str,
        orderType: str,
        side: str,
        time: int = generate_timestamp(),
        id: str = None,
        orderPrice: str = None,
        stopPrice: str = None,
        postOnly: bool = None,
        timeInForce: str = None,
        respInst: str = None,
    ):
        """
        :param product_symbol: str
        :param orderQty: str
        :param orderType: str ("market", "limit", "stop_market", "stop_limit")
        :param side: str ("buy", "sell")
        :param time: int (milliseconds)
        :param id: str (>=9 chars, optional)
        :param orderPrice: str (optional)
        :param stopPrice: str (optional)
        :param postOnly: bool (optional)
        :param timeInForce: str ("GTC", "IOC", "FOK")
        :param respInst: str ("ACK", "ACCEPT", "DONE")
        """
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.ASCENDEX, product_symbol),
            "orderQty": orderQty,
            "orderType": orderType,
            "side": side,
            "time": time,
        }
        if id is not None:
            payload["id"] = id
        if orderPrice is not None:
            payload["orderPrice"] = orderPrice
        if stopPrice is not None:
            payload["stopPrice"] = stopPrice
        if postOnly is not None:
            payload["postOnly"] = postOnly
        if timeInForce is not None:
            payload["timeInForce"] = timeInForce
        if respInst is not None:
            payload["respInst"] = respInst

        res = await self._request(
            method="POST",
            path=SpotOrder.PLACE_ORDER.route,
            hash_path=SpotOrder.PLACE_ORDER.hash,
            query=payload,
        )
        return res

    async def place_spot_market_order(
        self,
        product_symbol: str,
        side: str,
        orderQty: str,
        id: str = None,
        respInst: str = None,
    ):
        return await self.place_spot_order(
            product_symbol=product_symbol,
            orderQty=orderQty,
            orderType="market",
            side=side,
            id=id,
            respInst=respInst,
        )

    async def place_spot_market_buy_order(
        self,
        product_symbol: str,
        orderQty: str,
        id: str = None,
        respInst: str = None,
    ):
        return await self.place_spot_market_order(
            product_symbol=product_symbol,
            side="buy",
            orderQty=orderQty,
            id=id,
            respInst=respInst,
        )

    async def place_spot_market_sell_order(
        self,
        product_symbol: str,
        orderQty: str,
        id: str = None,
        respInst: str = None,
    ):
        return await self.place_spot_market_order(
            product_symbol=product_symbol,
            side="sell",
            orderQty=orderQty,
            id=id,
            respInst=respInst,
        )

    async def place_spot_limit_order(
        self,
        product_symbol: str,
        side: str,
        orderQty: str,
        orderPrice: str,
        id: str = None,
        timeInForce: str = "GTC",
        respInst: str = None,
    ):
        return await self.place_spot_order(
            product_symbol=product_symbol,
            orderQty=orderQty,
            orderType="limit",
            side=side,
            orderPrice=orderPrice,
            id=id,
            timeInForce=timeInForce,
            respInst=respInst,
        )

    async def place_spot_limit_buy_order(
        self,
        product_symbol: str,
        orderQty: str,
        orderPrice: str,
        id: str = None,
        timeInForce: str = "GTC",
        respInst: str = None,
    ):
        return await self.place_spot_limit_order(
            product_symbol=product_symbol,
            side="buy",
            orderQty=orderQty,
            orderPrice=orderPrice,
            id=id,
            timeInForce=timeInForce,
            respInst=respInst,
        )

    async def place_spot_limit_sell_order(
        self,
        product_symbol: str,
        orderQty: str,
        orderPrice: str,
        id: str = None,
        timeInForce: str = "GTC",
        respInst: str = None,
    ):
        return await self.place_spot_limit_order(
            product_symbol=product_symbol,
            side="sell",
            orderQty=orderQty,
            orderPrice=orderPrice,
            id=id,
            timeInForce=timeInForce,
            respInst=respInst,
        )

    async def place_spot_post_only_order(
        self,
        product_symbol: str,
        side: str,
        orderQty: str,
        orderPrice: str,
        id: str = None,
        respInst: str = None,
    ):
        return await self.place_spot_order(
            product_symbol=product_symbol,
            orderQty=orderQty,
            orderType="limit",
            side=side,
            orderPrice=orderPrice,
            postOnly=True,
            id=id,
            respInst=respInst,
        )

    async def place_spot_post_only_buy_order(
        self,
        product_symbol: str,
        orderQty: str,
        orderPrice: str,
        id: str = None,
        respInst: str = None,
    ):
        return await self.place_spot_post_only_order(
            product_symbol=product_symbol,
            side="buy",
            orderQty=orderQty,
            orderPrice=orderPrice,
            id=id,
            respInst=respInst,
        )

    async def place_spot_post_only_sell_order(
        self,
        product_symbol: str,
        orderQty: str,
        orderPrice: str,
        id: str = None,
        respInst: str = None,
    ):
        return await self.place_spot_post_only_order(
            product_symbol=product_symbol,
            side="sell",
            orderQty=orderQty,
            orderPrice=orderPrice,
            id=id,
            respInst=respInst,
        )

    async def cancel_spot_order(
        self,
        orderId: str,
        product_symbol: str,
        time: int = generate_timestamp(),
        id: str = None,
    ):
        """
        :param orderId: str
        :param product_symbol: str
        :param time: int
        :param id: str
        """
        payload = {
            "orderId": orderId,
            "symbol": self.ptm.get_exchange_symbol(Common.ASCENDEX, product_symbol),
            "time": time,
        }
        if id is not None:
            payload["id"] = id

        res = await self._request(
            method="DELETE",
            path=SpotOrder.CANCEL_ORDER.route,
            hash_path=SpotOrder.CANCEL_ORDER.hash,
            query=payload,
        )
        return res

    async def cancel_all_spot_orders(
        self,
        product_symbol: str,
    ):
        """
        :param product_symbol: str
        """
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.ASCENDEX, product_symbol),
        }

        res = await self._request(
            method="DELETE",
            path=SpotOrder.CANCEL_ALL_ORDERS.route,
            hash_path=SpotOrder.CANCEL_ALL_ORDERS.hash,
            query=payload,
        )
        return res

    async def place_spot_batch_orders(
        self,
        orders: list,
    ):
        """
        :param orders: list
        """
        payload = {
            "orders": orders,
        }

        res = await self._request(
            method="POST",
            path=SpotOrder.PLACE_BATCH_ORDERS.route,
            hash_path=SpotOrder.PLACE_BATCH_ORDERS.hash,
            query=payload,
        )
        return res

    async def cancel_spot_batch_orders(
        self,
        orders: list,
    ):
        """
        :param orders: list
        """
        payload = {
            "orders": orders,
        }

        res = await self._request(
            method="DELETE",
            path=SpotOrder.CANCEL_BATCH_ORDERS.route,
            hash_path=SpotOrder.CANCEL_BATCH_ORDERS.hash,
            query=payload,
        )
        return res

    async def get_order_status(
        self,
        orderId: str = None,
    ):
        """
        :param orderId: str
        """
        payload = {}
        if orderId is not None:
            payload["orderId"] = orderId

        res = await self._request(
            method="GET",
            path=SpotOrder.QUERY_ORDER.route,
            hash_path=SpotOrder.QUERY_ORDER.hash,
            query=payload,
        )
        return res

    async def get_list_open_orders(
        self,
        product_symbol: str,
    ):
        """
        :param product_symbol: str
        """
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.ASCENDEX, product_symbol),
        }

        res = await self._request(
            method="GET",
            path=SpotOrder.LIST_OPEN_ORDERS.route,
            hash_path=SpotOrder.LIST_OPEN_ORDERS.hash,
            query=payload,
        )
        return res

    async def get_list_order_history(
        self,
        product_symbol: str,
        account: str = "cash",
        start_time: int = None,
        end_time: int = None,
        seqNum: int = None,
        limit: int = None,
    ):
        """
        :param product_symbol: str
        :param account: str (cash, margin, future)
        :param start_time: int
        :param end_time: int
        :param seqNum: int
        :param limit: int
        """
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.ASCENDEX, product_symbol),
            "account": account,
        }
        if start_time is not None:
            payload["startTime"] = start_time
        if end_time is not None:
            payload["endTime"] = end_time
        if seqNum is not None:
            payload["seqNum"] = seqNum
        if limit is not None:
            payload["limit"] = limit

        res = await self._request(
            method="GET",
            path=SpotOrder.LIST_ORDER_HISTORY.route,
            hash_path=SpotOrder.LIST_ORDER_HISTORY.hash,
            query=payload,
        )
        return res



from ._http_manager import HTTPManager
from .endpoints.account import SpotAccount


class AccountHTTP(HTTPManager):
    async def get_account_balance(
        self,
        currency: str = None,
        type: str = None,
    ):
        payload = {}
        if currency:
            payload["currency"] = currency
        if type:
            payload["type"] = type

        res = await self._request(
            method="GET",
            path=SpotAccount.ACCOUNT_BALANCE,
            query=payload,
        )
        return res



from enum import Enum


class Market(str, Enum):
    GET_KLINE = "/api/v5/market/candles"
    GET_ORDERBOOK = "/api/v5/market/books"
    GET_TICKERS = "/api/v5/market/tickers"

    def __str__(self) -> str:
        return self.value



from logging.handlers import TimedRotatingFileHandler
import os
import time
from datetime import datetime, timezone


def generate_timestamp(iso_format=False):
    if iso_format:
        return datetime.now(timezone.utc).isoformat(timespec="milliseconds").replace("+00:00", "Z")
    return int(time.time() * 10**3)


def config_logging(logging, logging_level, log_file: str = None):
    """Configures logging with UTC timestamp and optional daily rotating log files.

    Example log format:
        2025-03-07 19:42:04.849 UTC DEBUG my_logger: Log message

    Args:
        logging: Python logging module
        logging_level (int/str): Log level (e.g., 10 for DEBUG, 20 for INFO)
        log_file (str, optional): Base filename for logs (e.g., "my_log.log"). If provided, logs will rotate daily.
    """

    # Set UTC time format
    logging.Formatter.converter = time.gmtime

    # Define log format
    log_format = "%(asctime)s.%(msecs)03d UTC %(levelname)s %(name)s: %(message)s"
    date_format = "%Y-%m-%d %H:%M:%S"

    # Create log formatter
    formatter = logging.Formatter(fmt=log_format, datefmt=date_format)

    # Configure logging to console
    logging.basicConfig(level=logging_level, format=log_format, datefmt=date_format)

    # If a log file is provided, enable daily log rotation
    if log_file:
        log_dir = os.path.dirname(log_file) or "."  # Ensure log directory exists
        os.makedirs(log_dir, exist_ok=True)

        # Create a rotating file handler (daily rotation)
        file_handler = TimedRotatingFileHandler(
            filename=log_file,  # This will be the base filename
            when="midnight",  # Rotate at midnight
            interval=1,  # Rotate every 1 day
            backupCount=30,  # Keep the last 7 days of logs
            encoding="utf-8",  # Support Unicode logs
            utc=True,  # Use UTC for time-based rotation
        )

        file_handler.setFormatter(formatter)
        file_handler.suffix = "%Y-%m-%d"  # Filename suffix pattern
        logging.getLogger().addHandler(file_handler)



from ._http_manager import HTTPManager
from .endpoints.account import CashAccount


class AccountHTTP(HTTPManager):
    async def get_account_info(self):
        payload = {}
        res = await self._request(
            method="GET",
            path=CashAccount.ACCOUNT_INFO,
            hash_path=CashAccount.ACCOUNT_INFO.hash,
            query=payload,
        )
        return res



# Import exchange client classes and create callable functions
from .bybit.client import Client as BybitClient
from .binance.client import Client as BinanceClient
from .okx.client import Client as OKXClient
from .bitmart.client import Client as BitmartClient
from .gateio.client import Client as GateioClient
from .hyperliquid.client import Client as HyperliquidClient
from .bingx.client import Client as BingXClient
from .kucoin.client import Client as KuCoinClient
from .ascendex.client import Client as AscendexClient
from .bitmex.client import Client as BitmexClient


async def bybit(**kwargs):
    """Create and initialize a Bybit client instance."""
    client = BybitClient(**kwargs)
    await client.async_init()
    return client


async def binance(**kwargs):
    """Create and initialize a Binance client instance."""
    client = BinanceClient(**kwargs)
    await client.async_init()
    return client


async def okx(**kwargs):
    """Create and initialize an OKX client instance."""
    client = OKXClient(**kwargs)
    await client.async_init()
    return client


async def bitmart(**kwargs):
    """Create and initialize a BitMart client instance."""
    client = BitmartClient(**kwargs)
    await client.async_init()
    return client


async def gateio(**kwargs):
    """Create and initialize a Gate.io client instance."""
    client = GateioClient(**kwargs)
    await client.async_init()
    return client


async def hyperliquid(**kwargs):
    """Create and initialize a Hyperliquid client instance."""
    client = HyperliquidClient(**kwargs)
    await client.async_init()
    return client


async def bingx(**kwargs):
    """Create and initialize a BingX client instance."""
    client = BingXClient(**kwargs)
    await client.async_init()
    return client


async def kucoin(**kwargs):
    """Create and initialize a KuCoin client instance."""
    client = KuCoinClient(**kwargs)
    await client.async_init()
    return client


async def ascendex(**kwargs):
    """Create and initialize a AscendEX client instance."""
    client = AscendexClient(**kwargs)
    await client.async_init()
    return client


async def bitmex(**kwargs):
    """Create and initialize a BitMEX client instance."""
    client = BitmexClient(**kwargs)
    await client.async_init()
    return client


__all__ = [
    "bybit",
    "binance",
    "okx",
    "bitmart",
    "gateio",
    "hyperliquid",
    "bingx",
    "kucoin",
    "ascendex",
    "bitmex",
]



import json
from krex.utils.common import Common
from ._http_manager import HTTPManager
from .endpoints.market import Market


class MarketHTTP(HTTPManager):
    async def get_instrument_info(
        self,
        product_symbol: str = None,
        filter: dict = None,
        count: int = None,
    ):
        """
        :param product_symbol: str
        :param filter: dict
        :param count: int
        """
        payload = {}
        if product_symbol is not None:
            payload["symbol"] = self.ptm.get_exchange_symbol(Common.BITMEX, product_symbol)
        if filter is not None:
            payload["filter"] = json.dumps(filter)
        if count is not None:
            payload["count"] = count

        res = await self._request(
            method="GET",
            path=Market.INSTRUMENT_INFO,
            query=payload,
            signed=False,
        )
        return res

    async def get_orderbook(
        self,
        product_symbol: str,
        depth: int = None,
    ):
        """
        :param product_symbol: str
        :param depth: int
        """
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.BITMEX, product_symbol),
        }
        if depth is not None:
            payload["depth"] = depth

        res = await self._request(
            method="GET",
            path=Market.ORDERBOOK,
            query=payload,
            signed=False,
        )
        return res

    async def get_trades(
        self,
        product_symbol: str = None,
        filter: dict = None,
        columns: str = None,
        count: int = None,
        start: int = None,
        reverse: bool = None,
        startTime: str = None,
        endTime: str = None,
    ):
        """
        :param product_symbol: str
        :param filter: dict
        :param columns: str
        :param count: int
        :param start: int
        :param reverse: bool
        :param startTime: str
        :param endTime: str
        """
        payload = {}
        if product_symbol is not None:
            payload["symbol"] = self.ptm.get_exchange_symbol(Common.BITMEX, product_symbol)
        if filter is not None:
            payload["filter"] = str(filter)
        if columns is not None:
            payload["columns"] = columns
        if count is not None:
            payload["count"] = count
        if start is not None:
            payload["start"] = start
        if reverse is not None:
            payload["reverse"] = reverse
        if startTime is not None:
            payload["startTime"] = startTime
        if endTime is not None:
            payload["endTime"] = endTime

        res = await self._request(
            method="GET",
            path=Market.TRADE,
            query=payload,
            signed=False,
        )
        return res

    async def get_ticker(
        self,
        binSize: str = None,
        partial: bool = None,
        symbol: str = None,
        filter: dict = None,
        columns: str = None,
        count: int = None,
        start: int = None,
        reverse: bool = None,
        startTime: str = None,
        endTime: str = None,
    ):
        """
        :param binSize: str
        :param partial: bool
        :param symbol: str
        :param filter: dict
        :param columns: str
        :param count: int
        :param start: int
        :param reverse: bool
        :param startTime: str
        :param endTime: str
        """
        payload = {}
        if binSize is not None:
            payload["binSize"] = binSize
        if partial is not None:
            payload["partial"] = partial
        if symbol is not None:
            payload["symbol"] = self.ptm.get_exchange_symbol(Common.BITMEX, symbol)
        if filter is not None:
            payload["filter"] = str(filter)
        if columns is not None:
            payload["columns"] = columns
        if count is not None:
            payload["count"] = count
        if start is not None:
            payload["start"] = start
        if reverse is not None:
            payload["reverse"] = reverse
        if startTime is not None:
            payload["startTime"] = startTime
        if endTime is not None:
            payload["endTime"] = endTime

        res = await self._request(
            method="GET",
            path=Market.TICKER,
            query=payload,
            signed=False,
        )
        return res

    async def get_kline(
        self,
        binSize: str = None,
        partial: bool = None,
        symbol: str = None,
        filter: dict = None,
        columns: str = None,
        count: int = None,
        start: int = None,
        reverse: bool = None,
        startTime: str = None,
        endTime: str = None,
    ):
        """
        :param binSize: str
        :param partial: bool
        :param symbol: str
        :param filter: dict
        :param columns: str
        :param count: int
        :param start: int
        :param reverse: bool
        :param startTime: str
        :param endTime: str
        """
        payload = {}
        if binSize is not None:
            payload["binSize"] = binSize
        if partial is not None:
            payload["partial"] = partial
        if symbol is not None:
            payload["symbol"] = self.ptm.get_exchange_symbol(Common.BITMEX, symbol)
        if filter is not None:
            payload["filter"] = str(filter)
        if columns is not None:
            payload["columns"] = columns
        if count is not None:
            payload["count"] = count
        if start is not None:
            payload["start"] = start
        if reverse is not None:
            payload["reverse"] = reverse
        if startTime is not None:
            payload["startTime"] = startTime
        if endTime is not None:
            payload["endTime"] = endTime

        res = await self._request(
            method="GET",
            path=Market.KLINE,
            query=payload,
            signed=False,
        )
        return res

    async def get_funding(
        self,
        product_symbol: str = None,
        filter: dict = None,
        columns: str = None,
        count: int = None,
        start: int = None,
        reverse: bool = None,
        startTime: str = None,
        endTime: str = None,
    ):
        """
        :param product_symbol: str
        :param filter: dict
        :param columns: str
        :param count: int
        :param start: int
        :param reverse: bool
        :param startTime: str
        :param endTime: str
        """
        payload = {}
        if product_symbol is not None:
            payload["symbol"] = self.ptm.get_exchange_symbol(Common.BITMEX, product_symbol)
        if filter is not None:
            payload["filter"] = str(filter)
        if columns is not None:
            payload["columns"] = columns
        if count is not None:
            payload["count"] = count
        if start is not None:
            payload["start"] = start
        if reverse is not None:
            payload["reverse"] = reverse
        if startTime is not None:
            payload["startTime"] = startTime
        if endTime is not None:
            payload["endTime"] = endTime

        res = await self._request(
            method="GET",
            path=Market.FUNDING,
            query=payload,
            signed=False,
        )
        return res



from enum import Enum


class FutureTrade(str, Enum):
    GET_ALL_POSITIONS = "/futures/{settle}/positions"
    GET_SINGLE_POSITION = "/futures/{settle}/positions/{contract}"
    UPDATE_POSITION_LEVERAGE = "/futures/{settle}/positions/{contract}/leverage"
    DUAL_MODE_SWITCH = "/futures/{settle}/dual_mode"

    # "POST" Create a new order, "GET" get list, "DELETE" cancel all orders
    FUTURES_ORDER = "/futures/{settle}/orders"
    BATCH_FUTURES_ORDERS = "/futures/{settle}/batch_orders"

    # "GET" get order, "POST" amend order, "DELETE" cancel single order
    SINGLE_ORDER = "/futures/{settle}/orders/{order_id}"
    LIST_PERSONAL_TRADING_HISTORY = "/futures/{settle}/my_trades"
    LIST_POSITION_CLOSE_HISTORY = "/futures/{settle}/position_close"
    LIST_AUTODELEVERAGING_HISTORY = "/futures/{settle}/auto_deleverages"

    def __str__(self) -> str:
        return self.value


class DeliveryTrade(str, Enum):
    GET_ALL_POSITIONS = "/delivery/{settle}/positions"
    GET_SINGLE_POSITION = "/delivery/{settle}/positions/{contract}"
    UPDATE_POSITION_LEVERAGE = "/delivery/{settle}/positions/{contract}/leverage"

    # "POST" Create a new order, "GET" get list, "DELETE" cancel all orders
    FUTURES_ORDER = "/delivery/{settle}/orders"

    # "GET" get order, "DELETE" cancel single order
    SINGLE_ORDER = "/delivery/{settle}/orders/{order_id}"
    LIST_PERSONAL_TRADING_HISTORY = "/delivery/{settle}/my_trades"
    LIST_POSITION_CLOSE_HISTORY = "/delivery/{settle}/position_close"


class SpotTrade(str, Enum):
    GET_OPEN_ORDER = "/spot/open_orders"
    # "POST" Create a new order, "GET" get list, "DELETE" cancel all orders
    SPOT_ORDER = "/spot/orders"

    # "GET" get order, "POST" amend order, "DELETE" cancel single order
    SINGLE_ORDER = "/spot/orders/{order_id}"
    LIST_PERSONAL_TRADING_HISTORY = "/spot/my_trades"

    def __str__(self) -> str:
        return self.value



from ._http_manager import HTTPManager
from .endpoint.path import Path
from .endpoint.asset import Asset


class AssetHTTP(HTTPManager):
    async def user_vault_equities(
        self,
        user: str,
    ):
        """
        :param user: str (waller address)
        """
        payload = {
            "type": Asset.USERVAULTEQUITIES,
            "user": user,
        }

        res = await self._request(
            method="POST",
            path=Path.INFO,
            query=payload,
            signed=False,
        )
        return res



from typing import Union
from ._http_manager import HTTPManager
from .endpoints.funds import Account


class AccountHTTP(HTTPManager):
    def get_wallet_summary(
        self,
        currency: str = "all",
        start_time: str = None,
        end_time: str = None,
        target_account_id: int = None,
        target_account_ids: Union[list, str] = None,
    ):
        """
        :param currency: str
        :param start_time: str
        :param end_time: str
        :param target_account_id: int
        :param target_account_ids: list[str] or str (can be "*")
        """
        payload = {
            "currency": currency,
        }

        if start_time is not None:
            payload["startTime"] = start_time

        if end_time is not None:
            payload["endTime"] = end_time

        if target_account_id is not None:
            payload["targetAccountId"] = target_account_id

        if target_account_ids is not None:
            if isinstance(target_account_ids, list):
                payload["targetAccountIds[]"] = target_account_ids
            elif isinstance(target_account_ids, str):
                payload["targetAccountIds"] = target_account_ids

        res = self._request(
            method="GET",
            path=Account.ACCOUNT_INFO,
            query=payload,
        )
        return res



import polars as pl
from typing import Any


def to_dataframe(data: Any, schema: list[str] = None) -> pl.DataFrame:
    if not data:
        return pl.DataFrame()

    if isinstance(data, list):
        if schema:
            return pl.DataFrame(data, schema=schema, orient="row")
        elif all(isinstance(item, dict) for item in data):
            return pl.DataFrame(data)
        else:
            return pl.DataFrame(data, orient="row")

    if isinstance(data, dict):
        return pl.DataFrame([data])

    return pl.DataFrame()



from dataclasses import dataclass
from ._market_http import MarketHTTP
from ._account_http import AccountHTTP
from ._trade_http import TradeHTTP
from ._position_http import PositionHTTP
from ._trading_http import TradingHTTP


@dataclass
class Client(
    MarketHTTP,
    AccountHTTP,
    TradeHTTP,
    PositionHTTP,
    TradingHTTP,
):
    def __init__(self, **args):
        super().__init__(**args)



from enum import Enum


class Trading(str, Enum):
    GET_EXECUTION_HISTORY = "/api/v1/user/executionHistory"
    GET_EXECUTIONS = "/api/v1/execution"
    GET_TRADE_HISTORY = "/api/v1/execution/tradeHistory"

    def __str__(self) -> str:
        return self.value



from .endpoints.trade import FuturesTrade, SpotTrade
from ._http_manager import HTTPManager
from .enums import BinanceExchangeType
from ...utils.common import Common
import time


class TradeHTTP(HTTPManager):
    async def set_leverage(
        self,
        product_symbol: str,
        leverage: int,
    ):
        """
        :param product_symbol: str
        :param leverage: int
        """
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.BINANCE, product_symbol),
            "leverage": leverage,
        }

        res = await self._request(
            method="POST",
            path=FuturesTrade.SET_LEVERAGE,
            query=payload,
        )
        return res

    async def place_order(
        self,
        product_symbol: str,
        side: str,
        type_: str,
        quantity: str = None,
        price: str = None,
        timeInForce: str = None,
        positionSide: str = None,
        reduceOnly: str = None,
        stopPrice: str = None,
        closePosition: str = None,
        activationPrice: str = None,
        callbackRate: str = None,
        workingType: str = None,
        priceProtect: str = None,
        newClientOrderId: str = None,
        newOrderRespType: str = None,
        priceMatch: str = None,
        selfTradePreventionMode: str = None,
        goodTillDate: int = None,
    ):
        """
        :param product_symbol: str
        :param side: str
        :param type_: str
        :param quantity: str
        :param price: str
        :param timeInForce: str
        :param positionSide: str
        :param reduceOnly: str
        :param stopPrice: str
        :param closePosition: str
        :param activationPrice: str
        :param callbackRate: str
        :param workingType: str
        :param priceProtect: str
        :param newClientOrderId: str
        :param newOrderRespType: str
        :param priceMatch: str
        :param selfTradePreventionMode: str
        :param goodTillDate: int
        """
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.BINANCE, product_symbol),
            "side": side,
            "type": type_,
        }

        if quantity is not None:
            payload["quantity"] = quantity
        if price is not None:
            payload["price"] = price
        if timeInForce is not None:
            payload["timeInForce"] = timeInForce
        if positionSide is not None:
            payload["positionSide"] = positionSide
        if reduceOnly is not None:
            payload["reduceOnly"] = reduceOnly
        if stopPrice is not None:
            payload["stopPrice"] = stopPrice
        if closePosition is not None:
            payload["closePosition"] = closePosition
        if activationPrice is not None:
            payload["activationPrice"] = activationPrice
        if callbackRate is not None:
            payload["callbackRate"] = callbackRate
        if workingType is not None:
            payload["workingType"] = workingType
        if priceProtect is not None:
            payload["priceProtect"] = priceProtect
        if newClientOrderId is not None:
            payload["newClientOrderId"] = newClientOrderId
        if newOrderRespType is not None:
            payload["newOrderRespType"] = newOrderRespType
        if priceMatch is not None:
            payload["priceMatch"] = priceMatch
        if selfTradePreventionMode is not None:
            payload["selfTradePreventionMode"] = selfTradePreventionMode
        if goodTillDate is not None:
            payload["goodTillDate"] = goodTillDate

        res = await self._request(
            method="POST",
            path=SpotTrade.PLACE_CANCEL_QUERY_ORDER
            if self.ptm.get_exchange_type(Common.BINANCE, product_symbol=product_symbol) == BinanceExchangeType.SPOT
            else FuturesTrade.PLACE_CANCEL_QUERY_ORDER,
            query=payload,
        )
        return res

    async def place_market_order(
        self,
        product_symbol: str,
        side: str,
        quantity: str,
        positionSide: str = None,
        reduceOnly: str = None,
    ):
        return await self.place_order(
            product_symbol=product_symbol,
            side=side,
            type_="MARKET",
            quantity=quantity,
            positionSide=positionSide,
            reduceOnly=reduceOnly,
        )

    async def place_market_buy_order(
        self,
        product_symbol: str,
        quantity: str,
        positionSide: str = None,
        reduceOnly: str = None,
    ):
        return await self.place_market_order(
            product_symbol=product_symbol,
            side="BUY",
            quantity=quantity,
            positionSide=positionSide,
            reduceOnly=reduceOnly,
        )

    async def place_market_sell_order(
        self,
        product_symbol: str,
        quantity: str,
        positionSide: str = None,
        reduceOnly: str = None,
    ):
        return await self.place_market_order(
            product_symbol=product_symbol,
            side="SELL",
            quantity=quantity,
            positionSide=positionSide,
            reduceOnly=reduceOnly,
        )

    async def place_limit_order(
        self,
        product_symbol: str,
        side: str,
        quantity: str,
        price: str,
        timeInForce: str = "GTC",
        positionSide: str = None,
        reduceOnly: str = None,
    ):
        return await self.place_order(
            product_symbol=product_symbol,
            side=side,
            type_="LIMIT",
            quantity=quantity,
            price=price,
            timeInForce=timeInForce,
            positionSide=positionSide,
            reduceOnly=reduceOnly,
        )

    async def place_limit_buy_order(
        self,
        product_symbol: str,
        quantity: str,
        price: str,
        timeInForce: str = "GTC",
        positionSide: str = None,
        reduceOnly: str = None,
    ):
        return await self.place_limit_order(
            product_symbol=product_symbol,
            side="BUY",
            quantity=quantity,
            price=price,
            timeInForce=timeInForce,
            positionSide=positionSide,
            reduceOnly=reduceOnly,
        )

    async def place_limit_sell_order(
        self,
        product_symbol: str,
        quantity: str,
        price: str,
        timeInForce: str = "GTC",
        positionSide: str = None,
        reduceOnly: str = None,
    ):
        return await self.place_limit_order(
            product_symbol=product_symbol,
            side="SELL",
            quantity=quantity,
            price=price,
            timeInForce=timeInForce,
            positionSide=positionSide,
            reduceOnly=reduceOnly,
        )

    async def place_post_only_limit_order(
        self,
        product_symbol: str,
        side: str,
        quantity: str,
        price: str,
        positionSide: str = None,
        reduceOnly: str = None,
    ):
        return await self.place_order(
            product_symbol=product_symbol,
            side=side,
            type_="LIMIT",
            quantity=quantity,
            price=price,
            timeInForce="GTX",  # GTX = Post Only
            positionSide=positionSide,
            reduceOnly=reduceOnly,
        )

    async def place_post_only_limit_buy_order(
        self,
        product_symbol: str,
        quantity: str,
        price: str,
        positionSide: str = None,
        reduceOnly: str = None,
    ):
        return await self.place_post_only_limit_order(
            product_symbol=product_symbol,
            side="BUY",
            quantity=quantity,
            price=price,
            positionSide=positionSide,
            reduceOnly=reduceOnly,
        )

    async def place_post_only_limit_sell_order(
        self,
        product_symbol: str,
        quantity: str,
        price: str,
        positionSide: str = None,
        reduceOnly: str = None,
    ):
        return await self.place_post_only_limit_order(
            product_symbol=product_symbol,
            side="SELL",
            quantity=quantity,
            price=price,
            positionSide=positionSide,
            reduceOnly=reduceOnly,
        )

    async def cancel_order(
        self,
        product_symbol: str,
        orderId: int = None,
        origClientOrderId: str = None,
    ):
        """
        :param product_symbol: str
        :param orderId: int
        :param origClientOrderId: str

        """
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.BINANCE, product_symbol),
        }
        if orderId is not None:
            payload["orderId"] = orderId
        if origClientOrderId is not None:
            payload["origClientOrderId"] = origClientOrderId

        res = await self._request(
            method="DELETE",
            path=SpotTrade.PLACE_CANCEL_QUERY_ORDER
            if self.ptm.get_exchange_type(Common.BINANCE, product_symbol=product_symbol) == BinanceExchangeType.SPOT
            else FuturesTrade.PLACE_CANCEL_QUERY_ORDER,
            query=payload,
        )
        return res

    async def get_order(
        self,
        product_symbol: str,
        orderId: int = None,
        origClientOrderId: str = None,
    ):
        """
        :param product_symbol: str
        :param orderId: int
        :param origClientOrderId: str

        """
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.BINANCE, product_symbol),
        }
        if orderId is not None:
            payload["orderId"] = orderId
        if origClientOrderId is not None:
            payload["origClientOrderId"] = origClientOrderId

        res = await self._request(
            method="GET",
            path=SpotTrade.PLACE_CANCEL_QUERY_ORDER
            if self.ptm.get_exchange_type(Common.BINANCE, product_symbol=product_symbol) == BinanceExchangeType.SPOT
            else FuturesTrade.PLACE_CANCEL_QUERY_ORDER,
            query=payload,
        )
        return res

    async def get_open_orders(
        self,
        product_symbol: str,
    ):
        payload = {}
        if product_symbol is not None:
            payload["symbol"] = self.ptm.get_exchange_symbol(Common.BINANCE, product_symbol)

        res = await self._request(
            method="GET",
            path=SpotTrade.QUERY_OPEN_ORDER
            if self.ptm.get_exchange_type(Common.BINANCE, product_symbol=product_symbol) == BinanceExchangeType.SPOT
            else FuturesTrade.QUERY_OPEN_ORDER,
            query=payload,
        )
        return res

    async def cancel_all_open_orders(
        self,
        product_symbol: str,
    ):
        """
        :param product_symbol: str

        """
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.BINANCE, product_symbol),
        }

        res = await self._request(
            method="DELETE",
            path=SpotTrade.CANCEL_ALL_ORDERS
            if self.ptm.get_exchange_type(Common.BINANCE, product_symbol=product_symbol) == BinanceExchangeType.SPOT
            else FuturesTrade.CANCEL_ALL_OPEN_ORDERS,
            query=payload,
        )
        return res

    async def get_future_all_order(
        self,
        product_symbol: str,
        orderId: int = None,
        startTime: int = None,
        endTime: int = None,
        limit: int = None,
    ):
        """
        :param product_symbol: str
        :param orderId: int
        :param startTime: int
        :param endTime: int
        :param limit: int
        """
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.BINANCE, product_symbol),
        }
        if orderId is not None:
            payload["orderId"] = orderId
        if startTime is not None:
            payload["startTime"] = startTime
        if endTime is not None:
            payload["endTime"] = endTime
        if limit is not None:
            payload["limit"] = limit

        res = await self._request(
            method="GET",
            path=FuturesTrade.QUERY_ALL_ORDERS,
            query=payload,
        )
        return res

    async def get_future_open_order(
        self,
        product_symbol: str,
        orderId: int = None,
        origClientOrderId: str = None,
    ):
        """
        :param product_symbol: str

        EitherorderId or origClientOrderId must be sent
        :param orderId: int
        :param origClientOrderId: str
        """
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.BINANCE, product_symbol),
        }
        if orderId is not None:
            payload["orderId"] = orderId
        if origClientOrderId is not None:
            payload["origClientOrderId"] = origClientOrderId

        res = await self._request(
            method="GET",
            path=FuturesTrade.QUERY_OPEN_ORDER,
            query=payload,
        )
        return res

    async def get_future_all_open_order(
        self,
        product_symbol: str,
    ):
        """
        :param product_symbol: str
        """
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.BINANCE, product_symbol),
        }

        res = await self._request(
            method="GET",
            path=FuturesTrade.QUERY_OPEN_ORDERS,
            query=payload,
        )
        return res

    async def get_future_position(
        self,
        product_symbol: str,
    ):
        """
        :param product_symbol: str
        """
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.BINANCE, product_symbol),
        }

        res = await self._request(
            method="GET",
            path=FuturesTrade.POSITION_INFO,
            query=payload,
        )
        return res

    """
    ---------------------------------------------------------
    """

    async def place_spot_order(
        self,
        product_symbol: str,
        side: str,
        type_: str,
        quantity: float = None,
        quoteOrderQty: float = None,
        price: float = None,
        timeInForce: str = None,
        newClientOrderId: str = None,
        strategyId: int = None,
        strategyType: int = None,
        stopPrice: float = None,
        trailingDelta: int = None,
        icebergQty: float = None,
        newOrderRespType: str = None,
        selfTradePreventionMode: str = None,
        recvWindow: int = None,
        timestamp: int = int(time.time() * 1000),
    ):
        """
        Place a spot order on Binance
        :param product_symbol: str
        :param side: str
        :param type_: str
        :param quantity: float
        :param quoteOrderQty: float
        :param price: float
        :param timeInForce: str
        :param newClientOrderId: str
        :param strategyId: int
        :param strategyType: int
        :param stopPrice: float
        :param trailingDelta: int
        :param icebergQty: float
        :param newOrderRespType: str
        :param selfTradePreventionMode: str
        :param recvWindow: int
        :param timestamp: int
        """
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.BINANCE, product_symbol),
            "side": side,
            "type": type_,
            "timestamp": timestamp,
        }

        # Required parameters based on order type
        if quantity is not None:
            payload["quantity"] = quantity
        if quoteOrderQty is not None:
            payload["quoteOrderQty"] = quoteOrderQty
        if price is not None:
            payload["price"] = price
        if timeInForce is not None:
            payload["timeInForce"] = timeInForce
        if newClientOrderId is not None:
            payload["newClientOrderId"] = newClientOrderId
        if strategyId is not None:
            payload["strategyId"] = strategyId
        if strategyType is not None:
            payload["strategyType"] = strategyType
        if stopPrice is not None:
            payload["stopPrice"] = stopPrice
        if trailingDelta is not None:
            payload["trailingDelta"] = trailingDelta
        if icebergQty is not None:
            payload["icebergQty"] = icebergQty
        if newOrderRespType is not None:
            payload["newOrderRespType"] = newOrderRespType
        if selfTradePreventionMode is not None:
            payload["selfTradePreventionMode"] = selfTradePreventionMode
        if recvWindow is not None:
            payload["recvWindow"] = recvWindow

        res = await self._request(
            method="POST",
            path=SpotTrade.PLACE_SPOT_ORDER,
            query=payload,
        )
        return res

    async def place_spot_market_order(
        self,
        product_symbol: str,
        side: str,
        quantity: float = None,
        quoteOrderQty: float = None,
        newClientOrderId: str = None,
        newOrderRespType: str = None,
    ):
        """
        Place a market order

        :param product_symbol: str - Trading pair symbol
        :param side: str - BUY or SELL
        :param quantity: float - Amount of base asset to buy/sell
        :param quoteOrderQty: float - Amount of quote asset to buy/sell
        :param newClientOrderId: str - Unique order ID
        :param newOrderRespType: str - Set response JSON: ACK, RESULT, or FULL
        """
        return await self.place_spot_order(
            product_symbol=product_symbol,
            side=side,
            type_="MARKET",
            quantity=quantity,
            quoteOrderQty=quoteOrderQty,
            newClientOrderId=newClientOrderId,
            newOrderRespType=newOrderRespType,
        )

    async def place_spot_market_buy_order(
        self,
        product_symbol: str,
        quantity: str = None,
        quoteOrderQty: str = None,
        newClientOrderId: str = None,
        newOrderRespType: str = None,
    ):
        """
        Place a market buy order

        :param product_symbol: str - Trading pair symbol
        :param quantity: str - Amount of base asset to buy
        :param quoteOrderQty: str - Amount of quote asset to spend
        :param newClientOrderId: str - Unique order ID
        :param newOrderRespType: str - Set response JSON: ACK, RESULT, or FULL
        """
        return await self.place_spot_market_order(
            product_symbol=product_symbol,
            side="BUY",
            quantity=quantity,
            quoteOrderQty=quoteOrderQty,
            newClientOrderId=newClientOrderId,
            newOrderRespType=newOrderRespType,
        )

    async def place_spot_market_sell_order(
        self,
        product_symbol: str,
        quantity: float = None,
        quoteOrderQty: float = None,
        newClientOrderId: str = None,
        newOrderRespType: str = None,
    ):
        """
        Place a market sell order

        :param product_symbol: str - Trading pair symbol
        :param quantity: str - Amount of base asset to sell
        :param newClientOrderId: str - Unique order ID
        :param newOrderRespType: str - Set response JSON: ACK, RESULT, or FULL
        """
        return await self.place_spot_market_order(
            product_symbol=product_symbol,
            side="SELL",
            quantity=quantity,
            quoteOrderQty=quoteOrderQty,
            newClientOrderId=newClientOrderId,
            newOrderRespType=newOrderRespType,
        )

    async def place_spot_limit_order(
        self,
        product_symbol: str,
        side: str,
        quantity: str,
        price: str,
        timeInForce: str = "GTC",
        newClientOrderId: str = None,
        newOrderRespType: str = None,
    ):
        """
        Place a limit order

        :param product_symbol: str - Trading pair symbol
        :param side: str - BUY or SELL
        :param quantity: str - Amount of base asset to buy/sell
        :param price: str - Order price
        :param timeInForce: str - GTC, IOC, FOK
        :param newClientOrderId: str - Unique order ID
        :param newOrderRespType: str - Set response JSON: ACK, RESULT, or FULL
        """
        return await self.place_spot_order(
            product_symbol=product_symbol,
            side=side,
            type_="LIMIT",
            quantity=quantity,
            price=price,
            timeInForce=timeInForce,
            newClientOrderId=newClientOrderId,
            newOrderRespType=newOrderRespType,
        )

    async def place_spot_limit_buy_order(
        self,
        product_symbol: str,
        quantity: str,
        price: str,
        timeInForce: str = "GTC",
        newClientOrderId: str = None,
        newOrderRespType: str = None,
    ):
        """
        Place a limit buy order

        :param product_symbol: str - Trading pair symbol
        :param quantity: str - Amount of base asset to buy
        :param price: str - Order price
        :param timeInForce: str - GTC, IOC, FOK
        :param newClientOrderId: str - Unique order ID
        :param newOrderRespType: str - Set response JSON: ACK, RESULT, or FULL
        """
        return await self.place_spot_limit_order(
            product_symbol=product_symbol,
            side="BUY",
            quantity=quantity,
            price=price,
            timeInForce=timeInForce,
            newClientOrderId=newClientOrderId,
            newOrderRespType=newOrderRespType,
        )

    async def place_spot_limit_sell_order(
        self,
        product_symbol: str,
        quantity: str,
        price: str,
        timeInForce: str = "GTC",
        newClientOrderId: str = None,
        newOrderRespType: str = None,
    ):
        """
        Place a limit sell order

        :param product_symbol: str - Trading pair symbol
        :param quantity: str - Amount of base asset to sell
        :param price: str - Order price
        :param timeInForce: str - GTC, IOC, FOK
        :param newClientOrderId: str - Unique order ID
        :param newOrderRespType: str - Set response JSON: ACK, RESULT, or FULL
        """
        return await self.place_spot_limit_order(
            product_symbol=product_symbol,
            side="SELL",
            quantity=quantity,
            price=price,
            timeInForce=timeInForce,
            newClientOrderId=newClientOrderId,
            newOrderRespType=newOrderRespType,
        )

    async def place_spot_post_only_limit_order(
        self,
        product_symbol: str,
        side: str,
        quantity: str,
        price: str,
        newClientOrderId: str = None,
        newOrderRespType: str = None,
    ):
        """
        Place a post-only limit order (LIMIT_MAKER)

        :param product_symbol: str - Trading pair symbol
        :param side: str - BUY or SELL
        :param quantity: str - Amount of base asset to buy/sell
        :param price: str - Order price
        :param newClientOrderId: str - Unique order ID
        :param newOrderRespType: str - Set response JSON: ACK, RESULT, or FULL
        """
        return await self.place_spot_order(
            product_symbol=product_symbol,
            side=side,
            type_="LIMIT_MAKER",
            quantity=quantity,
            price=price,
            newClientOrderId=newClientOrderId,
            newOrderRespType=newOrderRespType,
        )

    async def place_spot_post_only_limit_buy_order(
        self,
        product_symbol: str,
        quantity: str,
        price: str,
        newClientOrderId: str = None,
        newOrderRespType: str = None,
    ):
        """
        Place a post-only limit buy order

        :param product_symbol: str - Trading pair symbol
        :param quantity: str - Amount of base asset to buy
        :param price: str - Order price
        :param newClientOrderId: str - Unique order ID
        :param icebergQty: str - Used to create iceberg orders
        :param newOrderRespType: str - Set response JSON: ACK, RESULT, or FULL
        """
        return await self.place_spot_post_only_limit_order(
            product_symbol=product_symbol,
            side="BUY",
            quantity=quantity,
            price=price,
            newClientOrderId=newClientOrderId,
            newOrderRespType=newOrderRespType,
        )

    async def place_spot_post_only_limit_sell_order(
        self,
        product_symbol: str,
        quantity: str,
        price: str,
        newClientOrderId: str = None,
        newOrderRespType: str = None,
    ):
        """
        Place a post-only limit sell order

        :param product_symbol: str - Trading pair symbol
        :param quantity: str - Amount of base asset to sell
        :param price: str - Order price
        :param newClientOrderId: str - Unique order ID
        :param newOrderRespType: str - Set response JSON: ACK, RESULT, or FULL
        """
        return await self.place_spot_post_only_limit_order(
            product_symbol=product_symbol,
            side="SELL",
            quantity=quantity,
            price=price,
            newClientOrderId=newClientOrderId,
            newOrderRespType=newOrderRespType,
        )

    async def cancel_spot_order(
        self,
        product_symbol: str,
        orderId: int = None,
        origClientOrderId: str = None,
        timestamp: int = int(time.time() * 1000),
    ):
        """
        Cancel an active spot order on Binance.

        :param product_symbol: str - Trading pair symbol, e.g., 'BTCUSDT'
        :param orderId: int - Binance internal order ID (preferred for performance)
        :param origClientOrderId: str - The client order ID used when placing the order
        :param newClientOrderId: str - Unique ID for this cancel request (optional)
        :param cancelRestrictions: str - 'ONLY_NEW' or 'ONLY_PARTIALLY_FILLED' (optional)
        :param recvWindow: int - Optional timeout window in ms (max: 60000)
        :return: Binance API response
        """
        payload = {"symbol": self.ptm.get_exchange_symbol(Common.BINANCE, product_symbol), "timestamp": timestamp}

        # 至少要提供 orderId 或 origClientOrderId 其中之一
        if orderId is not None:
            payload["orderId"] = orderId
        elif origClientOrderId is not None:
            payload["origClientOrderId"] = origClientOrderId
        else:
            raise ValueError("Must provide either orderId or origClientOrderId to cancel an order.")

        # 發送 DELETE 請求
        res = await self._request(method="DELETE", path=SpotTrade.PLACE_SPOT_ORDER, query=payload)
        return res

    async def cancel_all_spot_orders(self, product_symbol: str, timestamp: int = int(time.time() * 1000)):
        """
        Cancel all active spot orders for a symbol on Binance.

        :param product_symbol: str - Trading pair symbol (e.g., "BTCUSDT")
        :param timestamp: int - Optional request timeout (max 60000)
        """
        payload = {"symbol": self.ptm.get_exchange_symbol(Common.BINANCE, product_symbol), "timestamp": timestamp}

        res = await self._request(
            method="DELETE",
            path=SpotTrade.CANCEL_ALL_SPOT_ORDERS,
            query=payload,
        )
        return res



from enum import Enum


class CashAccount(str, Enum):
    ACCOUNT_INFO = "/api/pro/v1/info"

    @property
    def hash(self):
        return self.value.split("/")[-1]

    def __str__(self) -> str:
        return self.value



from ._http_manager import HTTPManager
from .endpoints.trade import SwapTrade
from ...utils.common import Common


class TradeHTTP(HTTPManager):
    async def place_swap_order(
        self,
        product_symbol: str,
        type_: str,
        side: str,
        positionSide: str = None,
        reduceOnly: str = None,
        price: float = None,
        quantity: float = None,
        stopPrice: float = None,
        priceRate: float = None,
        stopLoss: str = None,
        takeProfit: str = None,
        workingType: str = None,
        clientOrderId: str = None,
        recvWindow: int = None,
        timeInForce: str = None,
        closePosition: str = None,
        activationPrice: float = None,
        stopGuaranteed: str = None,
        positionId: int = None,
    ):
        """
        :param product_symbol: str
        :param type_: str (MARKET, LIMIT, etc)
        :param side: str
        :param positionSide: str
        :param reduceOnly: str (true, false)
        :param price: float
        :param quantity: float
        :param stopPrice: float
        :param priceRate: float
        :param stopLoss: str
        :param takeProfit: str
        :param workingType: str
        :param clientOrderId: str
        :param recvWindow: int
        :param timeInForce: str
        :param closePosition: str (true, false)
        :param activationPrice: float
        :param stopGuaranteed: str
        :param positionId: int
        """
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.BINGX, product_symbol),
            "type": type_,
            "side": side,
        }
        if positionSide is not None:
            payload["positionSide"] = positionSide
        if reduceOnly is not None:
            payload["reduceOnly"] = reduceOnly
        if price is not None:
            payload["price"] = price
        if quantity is not None:
            payload["quantity"] = quantity
        if stopPrice is not None:
            payload["stopPrice"] = stopPrice
        if priceRate is not None:
            payload["priceRate"] = priceRate
        if stopLoss is not None:
            payload["stopLoss"] = stopLoss
        if takeProfit is not None:
            payload["takeProfit"] = takeProfit
        if workingType is not None:
            payload["workingType"] = workingType
        if clientOrderId is not None:
            payload["clientOrderId"] = clientOrderId
        if recvWindow is not None:
            payload["recvWindow"] = recvWindow
        if timeInForce is not None:
            payload["timeInForce"] = timeInForce
        if closePosition is not None:
            payload["closePosition"] = closePosition
        if activationPrice is not None:
            payload["activationPrice"] = activationPrice
        if stopGuaranteed is not None:
            payload["stopGuaranteed"] = stopGuaranteed
        if positionId is not None:
            payload["positionId"] = positionId

        res = await self._request(
            method="POST",
            path=SwapTrade.PLACE_ORDER,
            query=payload,
        )
        return res

    async def place_swap_market_order(
        self,
        product_symbol: str,
        side: str,
        quantity: float,
        clientOrderId: str = None,
        reduceOnly: bool = None,
        positionSide: str = None,
    ):
        return await self.place_swap_order(
            product_symbol=product_symbol,
            type_="MARKET",
            side=side,
            quantity=quantity,
            clientOrderId=clientOrderId,
            reduceOnly=reduceOnly,
            positionSide=positionSide,
        )

    async def place_swap_market_buy_order(
        self,
        product_symbol: str,
        quantity: float,
        positionSide: str = "LONG",
        clientOrderId: str = None,
        reduceOnly: bool = None,
    ):
        return await self.place_swap_market_order(
            product_symbol=product_symbol,
            side="BUY",
            quantity=quantity,
            positionSide=positionSide,
            clientOrderId=clientOrderId,
            reduceOnly=reduceOnly,
        )

    async def place_swap_market_sell_order(
        self,
        product_symbol: str,
        quantity: float,
        positionSide: str = "SHORT",
        clientOrderId: str = None,
        reduceOnly: bool = None,
    ):
        return await self.place_swap_market_order(
            product_symbol=product_symbol,
            side="SELL",
            quantity=quantity,
            positionSide=positionSide,
            clientOrderId=clientOrderId,
            reduceOnly=reduceOnly,
        )

    async def place_swap_limit_order(
        self,
        product_symbol: str,
        side: str,
        quantity: float,
        price: float,
        clientOrderId: str = None,
        timeInForce: str = "GTC",
        reduceOnly: bool = None,
        positionSide: str = None,
    ):
        return await self.place_swap_order(
            product_symbol=product_symbol,
            type_="LIMIT",
            side=side,
            quantity=quantity,
            price=price,
            clientOrderId=clientOrderId,
            timeInForce=timeInForce,
            reduceOnly=reduceOnly,
            positionSide=positionSide,
        )

    async def place_swap_limit_buy_order(
        self,
        product_symbol: str,
        quantity: float,
        price: float,
        positionSide: str = "LONG",
        timeInForce: str = "GTC",
        clientOrderId: str = None,
        reduceOnly: bool = None,
    ):
        return await self.place_swap_limit_order(
            product_symbol=product_symbol,
            side="BUY",
            quantity=quantity,
            price=price,
            positionSide=positionSide,
            timeInForce=timeInForce,
            clientOrderId=clientOrderId,
            reduceOnly=reduceOnly,
        )

    async def place_swap_limit_sell_order(
        self,
        product_symbol: str,
        quantity: float,
        price: float,
        positionSide: str = "SHORT",
        timeInForce: str = "GTC",
        clientOrderId: str = None,
        reduceOnly: bool = None,
    ):
        return await self.place_swap_limit_order(
            product_symbol=product_symbol,
            side="SELL",
            quantity=quantity,
            price=price,
            positionSide=positionSide,
            timeInForce=timeInForce,
            clientOrderId=clientOrderId,
            reduceOnly=reduceOnly,
        )

    async def place_swap_post_only_order(
        self,
        product_symbol: str,
        side: str,
        quantity: float,
        price: float,
        clientOrderId: str = None,
        timeInForce: str = "PostOnly",
        reduceOnly: str = None,
        positionSide: str = None,
    ):
        return await self.place_swap_order(
            product_symbol=product_symbol,
            type_="LIMIT",
            side=side,
            quantity=quantity,
            price=price,
            clientOrderId=clientOrderId,
            timeInForce=timeInForce,
            reduceOnly=reduceOnly,
            positionSide=positionSide,
        )

    async def place_swap_post_only_buy_order(
        self,
        product_symbol: str,
        quantity: float,
        price: float,
        positionSide: str = "LONG",
        clientOrderId: str = None,
        reduceOnly: str = None,
    ):
        return await self.place_swap_post_only_order(
            product_symbol=product_symbol,
            side="BUY",
            quantity=quantity,
            price=price,
            positionSide=positionSide,
            clientOrderId=clientOrderId,
            reduceOnly=reduceOnly,
        )

    async def place_swap_post_only_sell_order(
        self,
        product_symbol: str,
        quantity: float,
        price: float,
        positionSide: str = "SHORT",
        clientOrderId: str = None,
        reduceOnly: str = None,
    ):
        return await self.place_swap_post_only_order(
            product_symbol=product_symbol,
            side="SELL",
            quantity=quantity,
            price=price,
            positionSide=positionSide,
            clientOrderId=clientOrderId,
            reduceOnly=reduceOnly,
        )

    async def place_swap_batch_order(
        self,
        batchOrders: list,
    ):
        """
        :param batchOrders: list of order dicts
        """
        payload = {"batchOrders": batchOrders}
        res = await self._request(
            method="POST",
            path=SwapTrade.PLACE_BATCH_ORDER,
            query=payload,
        )
        return res

    async def cancel_swap_order(
        self,
        product_symbol: str,
        orderId: int = None,
        clientOrderId: str = None,
        recvWindow: int = None,
    ):
        """
        :param product_symbol: str
        :param orderId: int
        :param clientOrderId: str
        :param recvWindow: int
        """
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.BINGX, product_symbol),
        }
        if orderId is not None:
            payload["orderId"] = orderId
        if clientOrderId is not None:
            payload["clientOrderId"] = clientOrderId
        if recvWindow is not None:
            payload["recvWindow"] = recvWindow

        res = await self._request(
            method="DELETE",
            path=SwapTrade.CANCEL_ORDER,
            query=payload,
        )
        return res

    async def cancel_swap_batch_order(
        self,
        product_symbol: str,
        orderIdList: list = None,
        clientOrderIdList: list = None,
    ):
        """
        :param product_symbol: str
        :param orderIdList: list of orderId
        :param clientOrderIdList: list of clientOrderId
        """
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.BINGX, product_symbol),
        }
        if orderIdList is not None:
            payload["orderIdList"] = str(orderIdList).replace("'", "").replace(" ", "")
        if clientOrderIdList is not None:
            payload["clientOrderIdList"] = str(clientOrderIdList).replace("'", "").replace(" ", "")

        res = await self._request(
            method="DELETE",
            path=SwapTrade.CANCEL_BATCH_ORDER,
            query=payload,
        )
        return res

    async def cancel_swap_all_orders(
        self,
        product_symbol: str,
        type_: str = None,
    ):
        """
        :param product_symbol: str
        :param type_: str (LIMIT, MARKET)
        """
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.BINGX, product_symbol),
        }
        if type_ is not None:
            payload["type"] = type_

        res = await self._request(
            method="DELETE",
            path=SwapTrade.CANCEL_ALL_OPEN_ORDERS,
            query=payload,
        )
        return res

    async def replace_swap_order(
        self,
        product_symbol: str,
        orderId: str,
        cancelReplaceMode: str,
        type_: str,
        side: str,
        positionSide: str,
        cancelClientOrderId: int = None,
        cancelOrderId: str = None,
        cancelRestrictions: str = None,
        reduceOnly: str = None,
        price: float = None,
        quantity: float = None,
        stopPrice: float = None,
        priceRate: float = None,
        workingType: str = None,
        stopLoss: str = None,
        takeProfit: str = None,
        clientOrderId: str = None,
        closePosition: str = None,
        activationPrice: float = None,
        stopGuaranteed: str = None,
        timeInForce: str = None,
        positionId: int = None,
    ):
        """
        :param product_symbol: str
        :param orderId: str
        :param cancelReplaceMode: str
        :param type_: str
        :param side: str
        :param positionSide: str
        :param cancelClientOrderId: str
        :param cancelOrderId: str
        :param cancelRestrictions: str
        :param reduceOnly: bool (true, false)
        :param price: float
        :param quantity: float
        :param stopPrice: float
        :param priceRate: float
        :param workingType: str
        :param stopLoss: str
        :param takeProfit: str
        :param clientOrderId: str
        :param closePosition: str
        :param activationPrice: float
        :param stopGuaranteed: str
        :param timeInForce: str
        :param positionId: int
        """
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.BINGX, product_symbol),
            "orderId": orderId,
            "cancelReplaceMode": cancelReplaceMode,
            "type": type_,
            "side": side,
            "positionSide": positionSide,
        }
        if reduceOnly is not None:
            payload["reduceOnly"] = reduceOnly
        if price is not None:
            payload["price"] = price
        if quantity is not None:
            payload["quantity"] = quantity
        if cancelClientOrderId is not None:
            payload["cancelClientOrderId"] = cancelClientOrderId
        if cancelOrderId is not None:
            payload["cancelOrderId"] = cancelOrderId
        if cancelRestrictions is not None:
            payload["cancelRestrictions"] = cancelRestrictions
        if stopPrice is not None:
            payload["stopPrice"] = stopPrice
        if priceRate is not None:
            payload["priceRate"] = priceRate
        if workingType is not None:
            payload["workingType"] = workingType
        if stopLoss is not None:
            payload["stopLoss"] = stopLoss
        if takeProfit is not None:
            payload["takeProfit"] = takeProfit
        if clientOrderId is not None:
            payload["clientOrderId"] = clientOrderId
        if closePosition is not None:
            payload["closePosition"] = closePosition
        if activationPrice is not None:
            payload["activationPrice"] = activationPrice
        if stopGuaranteed is not None:
            payload["stopGuaranteed"] = stopGuaranteed
        if timeInForce is not None:
            payload["timeInForce"] = timeInForce
        if positionId is not None:
            payload["positionId"] = positionId

        res = await self._request(
            method="POST",
            path=SwapTrade.REPLACE_ORDER,
            query=payload,
        )
        return res

    async def close_swap_position(
        self,
        positionId: str,
    ):
        """
        :param positionId: str
        """
        payload = {
            "positionId": positionId,
        }

        res = await self._request(
            method="POST",
            path=SwapTrade.CLOSE_POSITION,
            query=payload,
        )
        return res

    async def close_swap_all_positions(
        self,
        product_symbol: str,
    ):
        """
        :param product_symbol: str
        """
        payload = {}
        if product_symbol:
            payload["symbol"] = self.ptm.get_exchange_symbol(Common.BINGX, product_symbol)

        res = await self._request(
            method="POST",
            path=SwapTrade.CLOSE_ALL_POSITIONS,
            query=payload,
        )
        return res

    async def get_order_detail(
        self,
        product_symbol: str,
        orderId: int = None,
        clientOrderId: str = None,
    ):
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.BINGX, product_symbol),
        }
        if orderId is not None:
            payload["orderId"] = orderId
        if clientOrderId is not None:
            payload["clientOrderId"] = clientOrderId

        res = await self._request(
            method="GET",
            path=SwapTrade.QUERY_ORDER_DETAIL,
            query=payload,
        )
        return res

    async def get_open_orders(
        self,
        product_symbol: str = None,
        type_: str = None,
    ):
        """
        :param product_symbol: str
        :param type_: str (LIMIT, MARKET)
        """
        payload = {}
        if product_symbol is not None:
            payload["symbol"] = self.ptm.get_exchange_symbol(Common.BINGX, product_symbol)
        if type_ is not None:
            payload["type"] = type_

        res = await self._request(
            method="GET",
            path=SwapTrade.QUERY_ALL_OPEN_ORDERS,
            query=payload,
        )
        return res

    async def get_order_history(
        self,
        product_symbol: str = None,
        currency: str = None,
        orderId: int = None,
        startTime: int = None,
        endTime: int = None,
        limit: int = None,
    ):
        """
        :param product_symbol: str
        :param currency: str
        :param orderId: int
        :param startTime: int
        :param endTime: int
        :param limit: int
        """
        payload = {}
        if product_symbol is not None:
            payload["symbol"] = self.ptm.get_exchange_symbol(Common.BINGX, product_symbol)
        if currency is not None:
            payload["currency"] = currency
        if orderId is not None:
            payload["orderId"] = orderId
        if startTime is not None:
            payload["startTime"] = startTime
        if endTime is not None:
            payload["endTime"] = endTime
        if limit is not None:
            payload["limit"] = limit

        res = await self._request(
            method="GET",
            path=SwapTrade.QUERY_ORDER_HISTORY,
            query=payload,
        )
        return res

    async def change_margin_type(
        self,
        product_symbol: str,
        marginType: str,
    ):
        """
        :param product_symbol: str
        :param marginType: str
        """
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.BINGX, product_symbol),
            "marginType": marginType,
        }

        res = await self._request(
            method="POST",
            path=SwapTrade.CHANGE_MARGIN_TYPE,
            query=payload,
        )
        return res

    async def get_margin_type(
        self,
        product_symbol: str = None,
    ):
        """
        :param product_symbol: str
        """
        payload = {}
        if product_symbol:
            payload["symbol"] = self.ptm.get_exchange_symbol(Common.BINGX, product_symbol)

        res = await self._request(
            method="GET",
            path=SwapTrade.QUERY_MARGIN_TYPE,
            query=payload,
        )
        return res

    async def set_leverage(
        self,
        product_symbol: str,
        side: str,
        leverage: int,
    ):
        """
        :param product_symbol: str
        :param side: str (LONG, SHORT)
        :param leverage: int
        """
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.BINGX, product_symbol),
            "side": side,
            "leverage": leverage,
        }

        res = await self._request(
            method="POST",
            path=SwapTrade.SET_LEVERAGE,
            query=payload,
        )
        return res

    async def get_leverage(
        self,
        product_symbol: str,
    ):
        """
        :param product_symbol: str
        """
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.BINGX, product_symbol),
        }

        res = await self._request(
            method="GET",
            path=SwapTrade.QUERY_LEVERAGE,
            query=payload,
        )
        return res

    async def set_position_mode(
        self,
        dualSidePosition: bool,
    ):
        """
        :param dualSidePosition: bool
        """
        payload = {
            "dualSidePosition": dualSidePosition,
        }

        res = await self._request(
            method="POST",
            path=SwapTrade.SET_POSITION_MODE,
            query=payload,
        )
        return res

    async def get_position_mode(
        self,
    ):
        payload = {}
        res = await self._request(
            method="GET",
            path=SwapTrade.QUERY_POSITION_MODE,
            query=payload,
        )
        return res



from ._http_manager import HTTPManager
from .endpoints.market import SpotMarket, FuturesMarket
from ...utils.common import Common
from ...utils.timeframe_utils import bitmart_convert_timeframe


class MarketHTTP(HTTPManager):
    async def get_spot_currencies(self):
        res = await self._request(
            method="GET",
            path=SpotMarket.GET_SPOT_CURRENCIES,
            query=None,
            signed=False,
        )
        return res

    async def get_trading_pairs(self):
        res = await self._request(
            method="GET",
            path=SpotMarket.GET_TRADING_PAIRS,
            query=None,
            signed=False,
        )
        return res

    async def get_trading_pairs_details(self):
        res = await self._request(
            method="GET",
            path=SpotMarket.GET_TRADING_PAIRS_DETAILS,
            query=None,
            signed=False,
        )
        return res

    async def get_ticker_of_all_pairs(self):
        res = await self._request(
            method="GET",
            path=SpotMarket.GET_TICKER_OF_ALL_PAIRS,
            query=None,
            signed=False,
        )
        return res

    async def get_ticker_of_a_pair(
        self,
        product_symbol: str,
    ):
        """
        :param product_symbol: str
        """
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.BITMART, product_symbol),
        }

        res = await self._request(
            method="GET",
            path=SpotMarket.GET_TICKER_OF_A_PAIR,
            query=payload,
            signed=False,
        )
        return res

    async def get_spot_kline(
        self,
        product_symbol: str,
        interval: str,
        before: int = None,
        after: int = None,
        limit: int = None,
    ):
        """
        :param product_symbol: str
        :param before: int
        :param after: int
        """
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.BITMART, product_symbol),
        }
        if interval is not None:
            payload["step"] = bitmart_convert_timeframe(interval)
        if before is not None:
            payload["before"] = before
        if after is not None:
            payload["after"] = after
        if limit is not None:
            payload["limit"] = limit

        res = await self._request(
            method="GET",
            path=SpotMarket.GET_SPOT_KLINE,
            query=payload,
            signed=False,
        )
        return res

    async def get_contracts_details(
        self,
        product_symbol: str = None,
    ):
        """
        :param product_symbol: str
        """
        payload = {}
        if product_symbol is not None:
            payload["symbol"] = self.ptm.get_exchange_symbol(Common.BITMART, product_symbol)

        res = await self._request(
            method="GET",
            path=FuturesMarket.GET_CONTRACTS_DETAILS,
            query=payload,
            signed=False,
        )
        return res

    async def get_depth(
        self,
        product_symbol: str,
    ):
        """
        :param product_symbol: str
        """
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.BITMART, product_symbol),
        }

        res = await self._request(
            method="GET",
            path=FuturesMarket.GET_DEPTH,
            query=payload,
            signed=False,
        )
        return res

    async def get_contract_kline(
        self,
        product_symbol: str,
        interval: str,
        start_time: int,
        end_time: int,
    ):
        """
        :param product_symbol: str
        :param startTime: int
        :param endTime: int
        """
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.BITMART, product_symbol),
            "step": bitmart_convert_timeframe(interval),
            "start_time": start_time,
            "end_time": end_time,
        }

        res = await self._request(
            method="GET",
            path=FuturesMarket.GET_CONTRACTS_KLINE,
            query=payload,
            signed=False,
        )
        return res

    async def get_current_funding_rate(
        self,
        product_symbol: str,
    ):
        """
        :param product_symbol: str
        """
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.BITMART, product_symbol),
        }

        res = await self._request(
            method="GET",
            path=FuturesMarket.GET_CURRENT_FUNDING_RATE,
            query=payload,
            signed=False,
        )
        return res

    async def get_funding_rate_history(
        self,
        product_symbol: str,
        limit: int = None,
    ):
        """
        :param product_symbol: str
        :param limit: int
        """
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.BITMART, product_symbol),
        }
        if limit is not None:
            payload["limit"] = limit

        res = await self._request(
            method="GET",
            path=FuturesMarket.GET_FUNDING_RATE_HISTORY,
            query=payload,
            signed=False,
        )
        return res



from krex.utils.timeframe_utils import kucoin_convert_timeframe
from ._http_manager import HTTPManager
from .endpoints.market import SpotMarket
from ...utils.common import Common


class MarketHTTP(HTTPManager):
    async def get_spot_instrument_info(
        self,
    ):
        payload = {}
        res = await self._request(
            method="GET",
            path=SpotMarket.INSTRUMENT_INFO,
            query=payload,
            signed=False,
        )
        return res

    async def get_spot_ticker(
        self,
        product_symbol: str,
    ):
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.KUCOIN, product_symbol),
        }

        res = await self._request(
            method="GET",
            path=SpotMarket.TICKER,
            query=payload,
            signed=False,
        )
        return res

    async def get_spot_all_tickers(
        self,
    ):
        payload = {}
        res = await self._request(
            method="GET",
            path=SpotMarket.ALL_TICKERS,
            query=payload,
            signed=False,
        )
        return res

    async def get_spot_orderbook(
        self,
        product_symbol: str,
    ):
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.KUCOIN, product_symbol),
        }

        res = await self._request(
            method="GET",
            path=SpotMarket.ORDERBOOK,
            query=payload,
        )
        return res

    async def get_spot_public_trades(
        self,
        product_symbol: str,
    ):
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.KUCOIN, product_symbol),
        }

        res = await self._request(
            method="GET",
            path=SpotMarket.PUBLIC_TRADES,
            query=payload,
            signed=False,
        )
        return res

    async def get_spot_kline(
        self,
        product_symbol: str,
        type_: str,
        startAt: int = None,
        endAt: int = None,
    ):
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.KUCOIN, product_symbol),
            "type": kucoin_convert_timeframe(type_),
        }

        if startAt is not None:
            payload["startAt"] = startAt
        if endAt is not None:
            payload["endAt"] = endAt

        res = await self._request(
            method="GET",
            path=SpotMarket.KLINE,
            query=payload,
            signed=False,
        )
        return res



import hmac
import time
import base64
import httpx
import logging
import json
import hashlib
from dataclasses import dataclass, field
from urllib.parse import urlencode

from krex.utils.common import Common
from ..product_table.manager import ProductTableManager
from ...utils.errors import FailedRequestError
from ...utils.helpers import generate_timestamp


def _sign(plain: bytes, key: bytes) -> str:
    """KuCoin signature generation using HMAC-SHA256"""
    hm = hmac.new(key, plain, hashlib.sha256)
    return base64.b64encode(hm.digest()).decode()


@dataclass
class HTTPManager:
    base_url: str = field(default="https://api.kucoin.com")
    api_key: str = field(default=None)
    api_secret: str = field(default=None)
    passphrase: str = field(default=None)
    timeout: int = field(default=10)
    logger: logging.Logger = field(default=None)
    session: httpx.AsyncClient = field(default=None, init=False)
    ptm: ProductTableManager = field(init=False)
    preload_product_table: bool = field(default=True)
    _encrypted_passphrase: str = field(default=None, init=False)

    async def async_init(self):
        """Initialize async HTTP manager"""
        self.session = httpx.AsyncClient(timeout=self.timeout)
        self._logger = self.logger or logging.getLogger(__name__)

        # Encrypt passphrase if credentials are provided
        if self.passphrase and self.api_secret:
            self._encrypted_passphrase = _sign(self.passphrase.encode("utf-8"), self.api_secret.encode("utf-8"))

        if self.preload_product_table:
            self.ptm = await ProductTableManager.get_instance(Common.KUCOIN)
        return self

    def _generate_headers(self, timestamp: str, signature: str) -> dict:
        """Generate headers for KuCoin API requests"""
        headers = {
            "Content-Type": "application/json",
        }

        if self.api_key and signature and self._encrypted_passphrase:
            headers.update(
                {
                    "KC-API-KEY": self.api_key,
                    "KC-API-SIGN": signature,
                    "KC-API-TIMESTAMP": timestamp,
                    "KC-API-PASSPHRASE": self._encrypted_passphrase,
                    "KC-API-KEY-VERSION": "2",
                }
            )

        return headers

    def _create_signature_payload(self, timestamp: str, method: str, path: str, body: str) -> str:
        """Create the payload for signature generation according to KuCoin API v2"""
        # For KuCoin API v2, the signature payload is: timestamp + method + path + body
        return f"{timestamp}{method.upper()}{path}{body}"

    async def _request(
        self,
        method: str,
        path: str,
        query: dict = None,
        signed: bool = True,
    ):
        """Make HTTP request to KuCoin API"""
        if not self.session:
            await self.async_init()

        # Prepare request data
        timestamp = str(int(time.time() * 1000))
        body = ""
        request_path = path
        signature_path = path

        # Handle different HTTP methods
        if method.upper() == "GET":
            if query:
                request_path = f"{path}?{urlencode(query)}"
                signature_path = f"{path}?{urlencode(query)}"  # GET: include query params in signature
        elif method.upper() in ["POST", "PUT"]:
            body = json.dumps(query) if query else ""
            # POST/PUT/DELETE: don't include query params in signature path
            signature_path = path
        elif method.upper() == "DELETE":
            if query:
                query_string = urlencode(query)
                request_path = f"{path}?{query_string}"
                signature_path = f"{path}?{query_string}"
            else:
                request_path = path
                signature_path = path

        # Generate signature if needed
        signature = ""
        if signed:
            if not (self.api_key and self.api_secret and self.passphrase):
                raise ValueError("Signed request requires API Key, Secret, and Passphrase.")

            # Create signature payload
            payload = self._create_signature_payload(timestamp, method, signature_path, body)
            signature = _sign(payload.encode("utf-8"), self.api_secret.encode("utf-8"))

        response = None
        try:
            url = f"{self.base_url}{request_path}"
            headers = self._generate_headers(timestamp, signature)

            if method.upper() == "GET":
                response = await self.session.get(url, headers=headers)
            elif method.upper() == "POST":
                response = await self.session.post(url, headers=headers, content=body)
            elif method.upper() == "PUT":
                response = await self.session.put(url, headers=headers, content=body)
            elif method.upper() == "DELETE":
                response = await self.session.delete(url, headers=headers)
            else:
                raise ValueError(f"Unsupported method: {method}")

            try:
                data = response.json()
            except Exception:
                data = {}

            timestamp = generate_timestamp(iso_format=True)

            # Check for KuCoin API errors
            if isinstance(data, dict) and data.get("code") != "200000":
                code = data.get("code", "Unknown")
                error_message = data.get("msg", "Unknown error")
                raise FailedRequestError(
                    request=f"{method} {url} | Body: {query}",
                    message=f"KUCOIN API Error: [{code}] {error_message}",
                    status_code=response.status_code,
                    time=timestamp,
                    resp_headers=response.headers,
                )

            # Check HTTP status code
            if not response.status_code // 100 == 2:
                raise FailedRequestError(
                    request=f"{method.upper()} {url} | Body: {query}",
                    message=f"HTTP Error {response.status_code}: {response.text}",
                    status_code=response.status_code,
                    time=timestamp,
                    resp_headers=response.headers,
                )

            return data

        except httpx.RequestError as e:
            raise FailedRequestError(
                request=f"{method.upper()} {url} | Body: {query}",
                message=f"Request failed: {str(e)}",
                status_code=response.status_code if response else "Unknown",
                time=timestamp,
                resp_headers=response.headers if response else None,
            )

    async def close(self):
        """Close the HTTP session"""
        if self.session:
            await self.session.aclose()



from enum import Enum


class Order(str, Enum):
    PLACE_ORDER = "/api/v2/order"
    AMEND_ORDER = "/api/v2/order"
    CANCEL_ORDER = "/api/v2/order"
    CANCEL_ALL_ORDERS = "/api/v2/order/all"
    QUERY_ORDER = "/api/v1/order"

    def __str__(self) -> str:
        return self.value



"""Helper functions for Jupyter Notebook compatibility."""

import asyncio
import sys
from typing import Optional


def is_jupyter_environment() -> bool:
    """
    Check if the code is running in a Jupyter Notebook or JupyterLab environment.

    Returns:
        bool: True if running in Jupyter, False otherwise
    """
    try:
        # 檢查是否有 IPython 核心
        from IPython import get_ipython

        ipython = get_ipython()

        if ipython is None:
            return False

        # 檢查是否在 notebook 或 lab 環境中
        if hasattr(ipython, "kernel"):
            return True

        # 檢查 IPython 的類名
        class_name = ipython.__class__.__name__
        if "ZMQ" in class_name or "Terminal" in class_name:
            return True

        return False
    except ImportError:
        return False


def has_running_event_loop() -> bool:
    """
    Check if there's a running asyncio event loop.

    Returns:
        bool: True if there's a running event loop, False otherwise
    """
    try:
        loop = asyncio.get_running_loop()
        return loop is not None
    except RuntimeError:
        return False


def ensure_nest_asyncio() -> Optional[str]:
    """
    Ensure that nest_asyncio is applied if running in a Jupyter environment with an active event loop.

    Returns:
        Optional[str]: A message indicating the result of the operation, or None if no action is needed.
    """
    # If not in Jupyter environment, no need to apply nest_asyncio
    if not is_jupyter_environment():
        return None

    # If there's no running event loop, no need to apply nest_asyncio
    if not has_running_event_loop():
        return None

    try:
        import nest_asyncio

        # Check if nest_asyncio is already applied
        if hasattr(asyncio, "_nest_patched"):
            return "nest_asyncio already applied"

        nest_asyncio.apply()

        return "nest_asyncio applied successfully for Jupyter environment"

    except ImportError:
        warning_msg = (
            "Warning: Running in Jupyter environment with active event loop, "
            "but nest_asyncio is not installed. "
            "Please install it with: pip install nest_asyncio"
        )
        print(warning_msg, file=sys.stderr)
        return warning_msg


def auto_apply_nest_asyncio(verbose: bool = False) -> None:
    """
    Automatically check and apply nest_asyncio if running in a Jupyter environment with an active event loop.

    Args:
        verbose (bool): If True, prints the result of the operation.
    """
    result = ensure_nest_asyncio()

    if verbose and result:
        print(f"[krex] {result}")


# Automatically apply nest_asyncio if in Jupyter environment with active event loop
auto_apply_nest_asyncio(verbose=False)



class APIRequestError(Exception):
    def __init__(self, request, message, status_code=None, time=None, resp_headers=None):
        self.request = request
        self.message = message
        self.status_code = status_code if status_code is not None else "Unknown"
        self.time = time if time is not None else "Unknown"
        self.resp_headers = resp_headers
        super().__init__(f"{message} (ErrCode: {self.status_code}) (ErrTime: {self.time})" f".\nRequest → {request}.")


class FailedRequestError(APIRequestError):
    pass


class InvalidRequestError(APIRequestError):
    pass


class APIException(Exception):
    def __init__(self, response):
        self.status_code = response.status_code
        self.response = response.text

    def __str__(self):
        return "APIException(http status=%s): response=%s" % (
            self.status_code,
            self.response,
        )


class RequestException(Exception):
    def __init__(self, message):
        self.message = message

    def __str__(self):
        return "RequestException: %s" % self.message


class ParamsException(Exception):
    def __init__(self, message):
        self.message = message

    def __str__(self):
        return "ParamsException: %s" % self.message



from enum import Enum


class Asset(str, Enum):
    CURRENCY_INFO = "/api/v5/asset/currencies"
    GET_BALANCES = "/api/v5/asset/balances"
    ASSET_VALUATION = "/api/v5/asset/asset-valuation"
    FUNDS_TRANSFER = "/api/v5/asset/transfer"
    TRANSFER_STATE = "/api/v5/asset/transfer-state"
    BILLS_INFO = "/api/v5/asset/bills"
    DEPOSIT_ADDRESS = "/api/v5/asset/deposit-address"
    DEPOSIT_HISTORY = "/api/v5/asset/deposit-history"
    WITHDRAWAL_COIN = "/api/v5/asset/withdrawal"
    CANCEL_WITHDRAWAL = "/api/v5/asset/cancel-withdrawal"
    WITHDRAWAL_HISTORY = "/api/v5/asset/withdrawal-history"
    GET_DEPOSIT_WITHDRAW_STATUS = "/api/v5/asset/deposit-withdraw-status"
    EXCHANGE_LIST = "/api/v5/asset/exchange-list"
    MONTHLY_STATEMENT = "/api/v5/asset/monthly-statement"
    GET_CURRENCIES = "/api/v5/asset/convert/currencies"
    CONVERT_HISTORY = "/api/v5/asset/convert/history"
